; ------------------------------------
; ACPTR
;
; Purpose: Get data from the serial bus
; Call address: $FFA5 (hex) 65445 (decimal)
; Communication registers: A
; Preparatory routines: TALK, TKSA
; Error returns: See READST
; Stack requirements: 13
; Registers affected: A, X
; ------------------------------------
ACPTR = $FFA5

; ------------------------------------
; CHKIN
;
; Purpose: Open a channel for input
; Call address: $FFC6 (hex) 65478 (decimal)
; Communication registers: X
; Preparatory routines: (OPEN)
; Error returns:
; Stack requirements: None
; Registers affected: A, X
; ------------------------------------
CHKIN = $FFC6

; ------------------------------------
; CHKOUT
;
; Purpose: Open a channel for output
; Call address: $FFC9 (hex) 65481 (decimal)
; Communication registers: X
; Preparatory routines: (OPEN)
; Error returns: 0,3,5,7 (See READST)
; Stack requirements: 4+
; Registers affected: A, X
; ------------------------------------
CHKOUT = $FFC9

; ------------------------------------
; CHRIN
;
; Purpose: Get a character from the input channel
; Call address: $FFCF (hex) 65487 (decimal)
; Communication registers: A
; Preparatory routines: (OPEN, CHKIN)
; Error returns: 0 (See READST)
; Stack requirements: 7+
; Registers affected: A, X
; ------------------------------------
CHRIN = $FFCF

; ------------------------------------
; CHROUT
;
; Purpose: Output a character
; Call address: $FFD2 (hex) 65490 (decimal)
; Communication registers: A
; Preparatory routines: (CHKOUT,OPEN)
; Error returns: 0 (See READST)
; Stack requirements: 8+
; Registers affected: A
; ------------------------------------
CHROUT = $FFD2

; ------------------------------------
; CIOUT
;
; Purpose: Transmit a byte over the serial bus
; Call address: $FFA8 (hex) 65448 (decimal)
; Communication registers: A
; Preparatory routines: LISTEN, [SECOND]
; Error returns: See READST
; Stack requirements: 5
; Registers affected: None
; ------------------------------------
CIOUT = $FFA8

; ------------------------------------
; CINT
;
; Purpose: Initialize screen editor & 6567 video chip
; Call address: $FF81 (hex) 65409 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns: None
; Stack requirements: 4
; Registers affected: A, X, Y
; ------------------------------------
CINT = $FF81

; ------------------------------------
; CLALL
;
; Purpose: Close all files
; Call address: $FFE7 (hex) 65511 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns: None
; Stack requirements: 11
; Registers affected: A, X
; ------------------------------------
CLALL = $FFE7

; ------------------------------------
; CLOSE
;
; Purpose: Close a logical file
; Call address: $FFC3 (hex) 65475 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: 0,240 (See READST)
; Stack requirements: 2+
; Registers affected: A, X, Y
; ------------------------------------
CLOSE = $FFC3

; ------------------------------------
; CLRCHN
;
; Purpose: Clear I/O channels
; Call address: $FFCC (hex) 65484 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns:
; Stack requirements: 9
; Registers affected: A, X
; ------------------------------------
CLRCHN = $FFCC

; ------------------------------------
; GETIN
;
; Purpose: Get a character
; Call address: $FFE4 (hex) 65508 (decimal)
; Communication registers: A
; Preparatory routines: CHKIN, OPEN
; Error returns: See READST
; Stack requirements: 7+
; Registers affected: A (X, Y)
; ------------------------------------
GETIN = $FFE4

; ------------------------------------
; IOBASE
;
; Purpose: Define I/O memory page
; Call address: $FFF3 (hex) 65523 (decimal)
; Communication registers: X, Y
; Preparatory routines: None
; Error returns:
; Stack requirements: 2
; Registers affected: X, Y
; ------------------------------------
IOBASE = $FFF3

; ------------------------------------
; IOINIT
;
; Purpose: Initialize I/O devices
; Call Address: $FF84 (hex) 65412 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns:
; Stack requirements: None
; Registers affected: A, X, Y
; ------------------------------------
IOINIT = $FF84

; ------------------------------------
; LISTEN
;
; Purpose: Command a device on the serial bus to listen
; Call Address: $FFB1 (hex) 65457 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: See READST
; Stack requirements: None
; Registers affected: A
; ------------------------------------
LISTEN = $FFB1

; ------------------------------------
; LOAD
;
; Purpose: Load RAM from device
; Call address: $FFD5 (hex) 65493 (decimal)
; Communication registers: A, X, Y
; Preparatory routines: SETLFS, SETNAM
; Error returns: 0,4,5,8,9, READST
; Stack requirements: None
; Registers affected: A, X, Y
; ------------------------------------
LOAD = $FFD5

; ------------------------------------
; MEMBOT
;
; Purpose: Set bottom of memory
; Call address: $FF9C (hex) 65436 (decimal)
; Communication registers: X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: None
; Registers affected: X, Y
; ------------------------------------
MEMBOT = $FF9C

; ------------------------------------
; MEMTOP
;
; Purpose: Set the top of RAM
; Call address: $FF99 (hex) 65433 (decimal)
; Communication registers: X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: X, Y
; ------------------------------------
MEMTOP = $FF99

; ------------------------------------
; OPEN
;
; Purpose: Open a logical file
; Call address: $FFC0 (hex) 65472 (decimal)
; Communication registers: None
; Preparatory routines: SETLFS, SETNAM
; Error returns: 1,2,4,5,6,240, READST
; Stack requirements: None
; Registers affected: A, X, Y
; ------------------------------------
OPEN = $FFC0

; ------------------------------------
; PLOT
;
; Purpose: Set cursor location
; Call address: $FFF0 (hex) 65520 (decimal)
; Communication registers: A, X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A, X, Y
; ------------------------------------
PLOT = $FFF0

; ------------------------------------
; RDTIM
;
; Purpose: Read system clock
; Call address: $FFDE (hex) 65502 (decimal)
; Communication registers: A, X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A, X, Y
; ------------------------------------
RDTIM = $FFDE

; ------------------------------------
; READST
;
; Purpose: Read status word
; Call address: $FFB7 (hex) 65463 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A
; ------------------------------------
READST = $FFB7

; ------------------------------------
; RESTOR
;
; Purpose: Restore default system and interrupt vectors
; Call address: $FF8A (hex) 65418 (decimal)
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A, X, Y
; ------------------------------------
RESTOR = $FF8A

; ------------------------------------
; SAVE
;
; Purpose: Save memory to a device
; Call address: $FFD8 (hex) 65496 (decimal)
; Communication registers: A, X, Y
; Preparatory routines: SETLFS, SETNAM
; Error returns: 5,8,9, READST
; Stack requirements: None
; Registers affected: A, X, Y
; BASIC TO MACHINE LANGUAGE   293
; ------------------------------------
SAVE = $FFD8

; ------------------------------------
; SCNKEY
;
; Purpose: Scan the keyboard
; Call address: $FF9F (hex) 65439 (decimal)
; Communication registers: None
; Preparatory routines: IOINIT
; Error returns: None
; Stack requirements: 5
; Registers affected: A, X, Y
; ------------------------------------
SCNKEY = $FF9F

; ------------------------------------
; SCREEN
;
; Purpose: Return screen format
; Call address: $FFED (hex) 65517 (decimal)
; Communication registers: X, Y
; Preparatory routines: None
; Stack requirements: 2
; Registers affected: X, Y
; ------------------------------------
SCREEN = $FFED

; ------------------------------------
; SECOND
;
; Purpose: Send secondary address for LISTEN
; Call address: $FF93 (hex) 65427 (decimal)
; Communication registers: A
; Preparatory routines: LISTEN
; Error returns: See READST
; Stack requirements: 8
; Registers affected: A
; ------------------------------------
SECOND = $FF93

; ------------------------------------
; SETLFS
;
; Purpose: Set up a logical file
; Call address: $FFBA (hex) 65466 (decimal)
; Communication registers: A, X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: None
; ------------------------------------
SETLFS = $FFBA

; ------------------------------------
; SETMSG
;
; Purpose: Control system message output
; Call address: $FF90 (hex) 65424 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A
; ------------------------------------
SETMSG = $FF90

; ------------------------------------
; SETNAM
;
; Purpose: Set file name
; Call address: $FFBD (hex) 65469 (decimal)
; Communication registers: A, X, Y
; Preparatory routines:
; Stack requirements: 2
; Registers affected:
; ------------------------------------
SETNAM = $FFBD

; ------------------------------------
; SETTIM
;
; Purpose: Set the system clock
; Call address: $FFDB (hex) 65499 (decimal)
; Communication registers: A, X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: None
; BASIC TO MACHINE LANGUAGE   299
; ------------------------------------
SETTIM = $FFDB

; ------------------------------------
; SETTMO
;
; Purpose: Set IEEE bus card timeout flag
; Call address: $FFA2 (hex) 65442 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: None
; +---------------------------------------------------------------------+
; | NOTE: This routine is used ONLY with an IEEE add-on card!           |
; +---------------------------------------------------------------------+
; ------------------------------------
SETTMO = $FFA2

; ------------------------------------
; STOP
;
; Purpose: Check if <STOP> key is pressed
; Call address: $FFE1 (hex) 65505 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: None
; Stack requirements: None
; Registers affected: A, X
; ------------------------------------
STOP = $FFE1

; ------------------------------------
; TALK
;
; Purpose: Command a device on the serial bus to TALK
; Call address: $FFB4 (hex) 65460 (decimal)
; Communication registers: A
; Preparatory routines: None
; Error returns: See READST
; Stack requirements: 8
; Registers affected: A
; ------------------------------------
TALK = $FFB4

; ------------------------------------
; TKSA
;
; Purpose: Send a secondary address to a device commanded to TALK
; Call address: $FF96 (hex) 65430 (decimal)
; Communication registers: A
; Preparatory routines: TALK
; Error returns: See READST
; Stack requirements: 8
; Registers affected: A
; 302   BASIC TO MACHINE LANGUAGE
; ------------------------------------
TKSA = $FF96

; ------------------------------------
; UDTIM
;
; Purpose: Update the system clock
; Call address: $FFEA (hex) 65514 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A, X
; ------------------------------------
UDTIM = $FFEA

; ------------------------------------
; UNLSN
;
; Purpose: Send an UNLISTEN command
; Call address: $FFAE (hex) 65454 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns: See READST
; Stack requirements: 8
; Registers affected: A
; ------------------------------------
UNLSN = $FFAE

; ------------------------------------
; UNTLK
;
; Purpose: Send an UNTALK command
; Call address: $FFAB (hex) 65451 (decimal)
; Communication registers: None
; Preparatory routines: None
; Error returns: See READST
; Stack requirements: 8
; Registers affected: A
; ------------------------------------
UNTLK = $FFAB

; ------------------------------------
; VECTOR
;
; Purpose: Manage RAM vectors
; Call address: $FF8D (hex) 65421 (decimal)
; Communication registers: X, Y
; Preparatory routines: None
; Error returns: None
; Stack requirements: 2
; Registers affected: A, X, Y
; ------------------------------------
VECTOR = $FF8D

; ----------------------------------------------------------------------
; References:
; made from programming reference via command:
;
; zcat c64reference.gz 
; | sed -n '/^  B-[0-9]/,/Description/ p'
; | sed -r -e '/^  B/ {s/.*Name: *([A-Z]*).*/\1/;h;s/(.*)/; ----\n; \1\n;/}' 
;          -e '/^[^a-zA-Z0-9-]*$/ d' 
;          -e '/^  / s/^ */; /' 
;          -e '/^; Call/ {G;h;s/.*: (.....).*/ = \1\n/;H;g;P;d}' 
;          -e '/^; Descr/ {s/^.*$/; ----/;G;P;s/[^\n]*\n[^\n]*\n//;s/\n//;}' 
; | sed -e 's/-/---------/g' 
;       -e 's/\r//g' > KERNAL.S


; vim: set syntax=asm6502 :
