
; *************************************
; *
; * ASCII and PETSCII tables
; *
; *
; * This file is part of CaTer.
; * (C) Lars Stollenwerk 2001 - 2013 
; *
; * CaTer is provided under the terms 
; * of GNU General Public License.
; *
; * For more information see the
; * README file.
; *
; *************************************

; -------------------------------------
; table ASCII to PETSCII 
;
; This tabel is used to convert incoming
; ASCII chars. 
;
; pet=$00 means ignore the char
; pet=$01 do something complicated: C
;         PETSCII and ASCII equal:  =
;         PETSCII and ASCII differ: /
; -------------------------------------
atp ;_0  _1  _2  _3  _4  _5  _6  _7  _8  _9  _a  _b  _c  _d  _e  _f

; --- Control chars ------------------------------------------------
;    NUL                     ACK BEL BS  TAB LF          CR     
;                                 C   C   C   C           C
.byt $00,$00,$00,$00,$00,$00,$00,$01,$01,$01,$01,$00,$00,$01,$00,$00  ; 0_
;                        NAK                     ESC            
;                                                 C
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00  ; 1_

; --- ASCII = PETSCII ----------------------------------------------
;    ' '  !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /   
.byt $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2a,$2b,$2c,$2d,$2e,$2f  ; 2_
;     0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?  
.byt $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3a,$3b,$3c,$3d,$3e,$3f  ; 3_

; --- upper case letters -------------------------------------------
;     @   A   B   C   D   E   F   G   H   I   J   K   L   M   N   O 
;     =   /   /   /   /   /   /   /   /   /   /   /   /   /   /   /
.byt $40,$c1,$c2,$c3,$c4,$c5,$c6,$c7,$c8,$c9,$ca,$cb,$cc,$cd,$ce,$cf  ; 4_
;     P   Q   R   S   T   U   V   W   X   Y   Z   [  \£   ]  ^↑  _← 
;     /   /   /   /   /   /   /   /   /   /   /   =   =   =   =   =
.byt $d0,$d1,$d2,$d3,$d4,$d5,$d6,$d7,$d8,$d9,$da,$5b,$5c,$5d,$5e,$5f  ; 5_

; --- lower case letters -------------------------------------------
;    `━   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o
;     /   /   /   /   /   /   /   /   /   /   /   /   /   /   /   /
.byt $c0,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4a,$4b,$4c,$4d,$4e,$4f  ; 6_
;     p   q   r   s   t   u   v   w   x   y   z  {╋  |▒  }┃  ~▒▒ DEL
;     /   /   /   /   /   /   /   /   /   /   /   /   /   /   /    
.byt $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5a,$db,$dc,$dd,$de,$00  ; 7_

;    --- upper control codes ---------------------------------------
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; 8_
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; 9_

;    --- latin 9 chars ---------------------------------------------
;    NBSP         £   €      (Š)  §  (š)  ©       «       -   
.byt $20,$df,$df,$bf,$be,$df,$d3,$b5,$53,$bb,$df,$bc,$df,$2d,$df,$df  ; a_
;     °   ±   ²   ³  (Ž)  µ          (ž)          »   Œ   œ  (Ÿ)  
.byt $ba,$b8,$b6,$b7,$da,$b9,$df,$df,$5a,$df,$df,$bd,$b0,$b0,$d9,$df  ; b_
;     À  (Á)  Â  (Ã)  Ä   Å      (Ç)  È   É   Ê  (Ë) (Ì) (Í) (Î) (Ï)
.byt $a5,$c1,$a4,$c1,$a3,$a4,$df,$c3,$ad,$ab,$ac,$c5,$c9,$c9,$c9,$c9  ; c_
;    (Ð) (Ñ) (Ò) (Ó)  Ô  (Õ)  Ö      (Ø) (Ù) (Ú) (Û)  Ü  (Ý)      ß
.byt $c4,$ce,$cf,$cf,$b1,$cf,$af,$df,$cf,$d5,$d5,$d5,$b3,$d9,$df,$b4  ; d_
;     à  (á)  â  (ã)  ä   å       ç   è   é   ê   ë  (ì) (í) (î) (ï)
.byt $a2,$41,$a1,$41,$a0,$a1,$df,$a6,$aa,$a8,$a9,$a7,$49,$49,$49,$49  ; e_
;        (ñ) (ò) (ó)  ô  (õ)  ö      (ø) (ù) (ú) (û)  ü  (ý)     (ÿ)
.byt $df,$4e,$4f,$4f,$b1,$4f,$ae,$df,$4f,$55,$55,$55,$b2,$59,$df,$59  ; f_


; -------------------------------------
; table PETSCII to ASCII 
;
; This table is used to prepare keyboard
; input for sending over the serial 
; line.
;
; ascii = $00 means ignore key
; ascii = $ff menas send string
; ascii = $fe means do something 
;             complicated (command key)
; -------------------------------------
pta ;_0  _1  _2  _3  _4  _5  _6  _7  _8  _9  _a  _b  _c  _d  _e  _f

; --- Control chars ------------------------------------------------
;               {STOP}                                {RETURN}
;        ^A  ^B  ^C  ^D  ^E  ^F  ^G  ^H  ^I  ^J  ^K  ^L  ^M  ^N  ^O
.byt $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0a,$0b,$0c,$0d,$0e,$0f  ; 0_
;      {crsr↓} {HOME|DEL}                              {crsr→}
;    ^P  ^Q  ^R  ^S  ^T  ^U  ^V  ^W  ^X  ^Y  ^Z  ^[  ^\£ ^]  ^↑
.byt $10,$fe,$12,$fe,$fe,$15,$16,$17,$18,$19,$1a,$1b,$1c,$fe,$1e,$1f  ; 1_

; --- ASCII = PETSCII ----------------------------------------------
;    ' '  !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /   
.byt $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2a,$2b,$2c,$2d,$2e,$2f  ; 2_
;     0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?  
.byt $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3a,$3b,$3c,$3d,$3e,$3f  ; 3_

; --- lower case letters -------------------------------------------
;     @   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o
;     =   /   /   /   /   /   /   /   /   /   /   /   /   /   /   /
.byt $40,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6a,$6b,$6c,$6d,$6e,$6f  ; 4_
;     p   q   r   s   t   u   v   w   x   y   z   [  \£   ]  ^↑   ← 
;     /   /   /   /   /   /   /   /   /   /   /   =   =   =   =  ESC
.byt $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7a,$5b,$5c,$5d,$5e,$1b  ; 5_

; --- mirror of upper letters - should never appear ----------------
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; 6_
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; 7_

; --- upper control chars ------------------------------------------
;                        {f1}{f3}{f5}{f7}{f2}{f4}{f6}{f8}{ShRET}
;
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; 8_
;      {crsr↑} {CLR}{INS}                              {crsr←}
;                    DEL
.byt $00,$ff,$00,$00,$7f,$00,$00,$00,$00,$00,$00,$00,$00,$ff,$00,$00  ; 9_

; --- block graphics -----------------------------------------------
;   ShSP C=K C=I C=T C=@ C=G C=+ C=M C=£ Sh£ C=N C=Q C=D C=Z C=S C=P
;                                         |
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$7c,$00,$fe,$fe,$00,$fe,$00  ; a_
;    C=A C=E C=R C=W C=H C=J C=L C=Y C=U C=O Sh@ C=F C=C C=X C=V C=B
;                                             `
.byt $fe,$00,$fe,$00,$fe,$00,$fe,$00,$00,$00,$60,$fe,$fe,$00,$00,$00  ; b_

; --- capital letters ----------------------------------------------
;    Sh*  A   B   C   D   E   F   G   H   I   J   K   L   M   N   O 
;     _   /   /   /   /   /   /   /   /   /   /   /   /   /   /   /
.byt $5f,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4a,$4b,$4c,$4d,$4e,$4f  ; c_
;     P   Q   R   S   T   U   V   W   X   Y   Z  Sh+ C=- Sh-  π  C=*
;     /   /   /   /   /   /   /   /   /   /   /   {       }   ~    
.byt $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5a,$7b,$00,$7d,$7e,$00  ; d_

; --- mirror of block graphics - should never appear ---------------
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; e_
.byt $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; f_

; vim: set syntax=asm6502 :
