; *************************************
; *                                   *
; *             C a T e r             *
; *                                   *
; *           Copyright by            *
; *         Lars Stollenwerk          *
; *                                   *
; *           2001 - 2003             *
; *                                   *
; * This file is part of CaTer.       *
; *                                   *
; * CaTer is provided under the terms *
; * of GNU General Public License.    *
; *                                   *
; * For more information see the      *
; * README file.                      *
; *                                   *
; *************************************

; *************************************
; *
; * Constant declaration
; *
; *************************************

; -------------------------------------
; Configuration Constants
;
; -------------------------------------
; The use of configuration values is 
; labeled with '_c', so they can be
; changed in the binary file.

; --- colour values ---
; 0 black       8 orange
; 1 white       9 brown
; 2 red         a light red
; 3 cyan        b gray 1
; 4 purple      c gray 2
; 5 green       d light green
; 6 blue        e light blue
; 7 yellow      f gray 3

; --- font type ---
; vanilla     f  bold 1
; underline   e       3
fVa = $0f
fBo = $01
fUl = $0e
fUlBo = $03

; text background 
bgcolour = $0b      ; gray1

; border colour 
bgocolour = $0c     ; gray2

; color for cater speaking 
; (xmodem, floppy)
colExtra = $07

; Number panel backgroung
bgNumber = $07

; Number panel foreground
colNumber = $00

; --- bell sound ---

; activate buzzer
RBuzz = $80

; activate bell
RBell = $81

; --- initial baudrate ---
; $06 = 300
; $08 = 1200
; $0a = 2400
; $0c = 9600
; $0e = iec
InitBaud = $0c

; --- initial floppy device numer ---
Fln = 8     

; --- initial IEC connection ---
InitIECdev = 9

; *************************************
; *
; * Main Program
; *
; *************************************

; -------------------------------------
; cold start -- init
;
; -------------------------------------

; do what KERNAL does
start	stx $d016
	jsr $fda3
	jsr $fd50
	jsr $fd15
	jsr $ff5b
	cli
        
; init terminal      
        jsr InitVar ; memory variables
        jsr InitIRQ
        jsr InitChar; init font
        jsr InitSprites
        jsr InitBell; sid, bell & buzzer
        jsr InitScr ; init screen
        jsr InitCh  ; init channels

; -------------------------------------
; main loop
;
; -------------------------------------

; read rs232
main    
        jsr getRS
        bcs main2   ; no valid char
        jsr ProcIn  
        jmp main
        
; read keyboard
main2   
        jsr GetChr 
        cmp #$00
        beq main    ; no key pressed
        tay         ; key to Y
        jsr ProcOut
        jmp main

; *************************************
; *
; * imcoming data
; *
; *************************************

; -------------------------------------
; process incoming data
;
; char in Y
; -------------------------------------

ProcIn  lda DMode   ; handle dump mode
        bne PIDump
        lda EMode   ; handle esc mode
        bne PIEsc
        lda Xmodem  ; handle XMODEM send
        beq PI1
        cpy #NAK    ; got NAK?
        bne PI1     ; no -> go on
        jmp XsendNAK  ; start sender
PI1     lda atp,y   ; ASCII to PETSCII
        beq PIrts   ; ignore $00       
        cmp #$01    ; something special?
        beq Special        
        jsr CPrnChr ; print to screen        
PIrts   rts

; to far for branch
PIEsc   jmp Esc
PIDump  jmp Dump

; -------------------------------------
; special incoming char
;
; -------------------------------------

Special tya         ; restore char
; --- CR ---
        cmp #$0d    ; CR?
        bne D1
CR      jsr clPending
	ldx sRow    ; get row
        ldy #$00    ; set col=0
        jsr CPlot   ; set crsr
        rts
; --- BS ---
D1      cmp #$08    ; BS?
        bne D2
BS	jsr clPending
	ldy sCol    ; get col
        beq D1rts   ; stop @ left margin
        dey         ; dec column
        ldx sRow    ; get row
        jsr CPlot   ; set crsr
D1rts   rts
; --- ESC ---
D2      cmp #$1b    ; esc?
        bne D3
        lda #$0f    ; set esc mode
        sta EMode
        rts
; --- BEL ---
D3      cmp #$07    ; BEL?
        bne D4
        lda #RBell  ; request bell
        sta BellR
        rts
; --- LF ---
D4      cmp #$0a    ; LF?
        bne D5
LF      jsr clPending
	ldx sRow    ; crsr line
        cpx SRE     ; end scroll region?
        bne D4a     ;  no -> branch
        jsr CUScrl  ;  yes -> scroll up
        rts
D4a     cpx #$18    ; end of screen?
        bne D4b     ;  no -> branch
        rts         ;  yes -> do nothing
D4b     inx         ; next line
        ldy sCol    ; get col
        jsr CPlot   ; set crsr
        rts        
; --- TAB ---
D5      cmp #$09    ; TAB?
        bne D6
	; don't clear pending
	; don't set pending
        lda sCol    ; crsr col
        and #$f8    ; (col DIV 8) * 8
        clc         ; col + 8
        adc #$08
        cmp #$28    ; col=40?
        bne D5a     ; no -> skip
        lda #$27    ; yes -> col=39
D5a     tay         ; col to y
        ldx sRow    ; line to x
        jsr CPlot   ; set crsr
        rts

D6      rts        

; -------------------------------------
; Dump - print char and hex value 
;
; params: char in Y
; affects: A, X
; -------------------------------------
Dump    jsr COff
        ; -- print char --
	lda atp,y     ; ASCII to PETSCII
        jsr PrnChr
        ; -- reverse hex code --
        ldx #$01      ; revers on
        stx Rvs
        tya           ; restore char
	sec	      ; c=1: hi nibble
        jsr HexCode
        jsr PrnChr
        tya           ; restore char
        clc	      ; c=0: lo nibble
        jsr HexCode
        jsr PrnChr
        ldx #$00      ; revers off
        stx Rvs
        ; -- space --
        lda #$20      ; print space
        jsr PrnChr    
        
        jsr COn
        rts

; -------------------------------------
; esc mode
;
; char in Y
; EMode != $00 in A
; -------------------------------------

Esc     tax         ; save EMode
        and #$0f    ; EMode = $0f?
        bne E1

; --- throw mode --- EMode = $f0
;     throw away char
        lda #$00    ; reset EMode
        sta EMode
        rts

E1      txa         ; restore EMode
        and #$f0    ; EMode = $ff?
        beq SEsc    ; no -> short Emode
        jmp LEsc    ; yes -> long Emode
        
; -------------------------------------
; short esc mode
;
; EMode = $0f
; process first char
; -------------------------------------
SEsc    tya         ; restore char
	; --- [ ---
        cmp #$5b  ; [ ?
        bne E2
        lda #$ff    ; set esc [ mode
        sta EMode
        rts
	; --- ( ---
E2      cmp #$28    ; ( ?
        bne E3
        jmp sThrow
	; --- ) ---
E3      cmp #$29    ; ) ?
        bne E4
        jmp sThrow
	; --- # ---
E4      cmp #$23    ; # ?
        bne E5
        jmp sThrow
	; --- D --- index 
E5      cmp #$44    ; D ?
        bne E6
        jsr LF      ; same as LF
        jmp Eend
	; --- M --- reverse index
E6      cmp #$4d    ; M ?
        bne E7
	jsr clPending
        ldx sRow    ; get crsr row
        cpx SRS     ; top os scroll reg?
        bne E6a
        jsr CDScrl  ; yes -> scroll down
        jmp Eend
E6a     cpx #$00    ; top of screen?
        bne E6b
        jmp Eend    ; yes -> do nothing
E6b     dex         ; one line up
        ldy sCol    ; get crsr col
        jsr CPlot   ; set crsr
        jmp Eend
	; --- E --- next line
E7      cmp #$45    ; E ?
        bne E8
        jsr CR
        jsr LF
        jmp Eend
	; --- 7 --- save crsr
E8      cmp #$37    ; 7 ?
        bne E9
        lda Font    ; save font
        sta SaveF
        lda Rvs     ; save reverse mode
        sta SaveR
        ldx sRow    ; save position
        ldy sCol
        stx SaveRow
        sty SaveCol
        jmp Eend
	; --- 8 --- restore crsr
E9      cmp #$38    ; 8 ?
        bne E10
	jsr clPending
        ldx SaveRow ; restore pos
        ldy SaveCol
        jsr CPlot
        lda SaveR   ; restore ..
        sta Rvs     ; .. reverse mode
        ldx SaveF   ; restore font
        stx Font
        lda FontTable,x
        sta sColor  ; set colour
        jmp Eend

	; --- unknown ---
E10        

	; --- reset ESC mode ---
Eend    lda #$00    ; reset EMode
        sta EMode
        rts 

	; --- set Throw mode ---
sThrow  lda #$f0    ; set esc mode $f0
        sta EMode
        rts

; -------------------------------------
; [ esc mode
;
; EMode = $ff
; -------------------------------------
  
LEsc    tya         ; restore char

        ldy EBufL
        sta EBuf,y  ; store char
        iny
        sty EBufL   ; inc esc buffer
        
        jsr TestL   ; test letter
        bcs LE1     ; process command
        rts

; --- process esc command ---
; A = last char
; Y = EBufL
; X conunts processed command chars
LE1     ldx #$00    ; first char 

; --- A --- crsr up       
        cmp #$41    ; A ?
        bne LE2
	jsr clPending
        jsr GetNum  ; get argument
        lda EPar    ; EPar = 0...
        bne LE1c
        inc EPar    ; .. means 1
LE1c    lda sRow    ; get crsr row        
        sec
        sbc EPar    ; row = row - up
        cmp SRS     ; stop at top of ..
        bpl LE1a    ; ..scroll region
        lda SRS    
LE1a    tax         ; x is row
        ldy sCol    ; y is col
        jsr CPlot   ; set crsr
        jmp LEend
        
; --- B --- crsr down
LE2     cmp #$42    ; B ?
        bne LE3
	jsr clPending
        jsr GetNum  ; get argument
        lda EPar    ; EPar = 0...
        bne LE2c
        inc EPar    ; .. means 1        
LE2c    lda sRow    ; get crsr row        
        clc
        adc EPar    ; row = row + down
        cmp SRE     ; outside scrregion?
        bcs LE2d    ; yes -> branch
        tax         ; x is row
        jmp LE2a
LE2d    ldx SRE     ; x = row = SRE   
LE2a    ldy sCol    ; y is col
        jsr CPlot   ; set crsr
        jmp LEend

; --- C --- crsr right
LE3     cmp #$43    ; C ?
        bne LE4
	jsr clPending
        jsr GetNum  ; get argument        
        lda EPar    ; EPar = 0...
        bne LE3c
        inc EPar    ; .. means 1
LE3c    lda sCol    ; get crsr col        
        clc
        adc EPar    ; col = col + right
        cmp #$27    ; outside screen?
        bcs LE3d    ; yes -> branch
        tay
        jmp LE3a
LE3d    ldy #$27    ; y=col=left margin
LE3a    ldx sRow    ; x is row
        jsr CPlot   ; set crsr
        jmp LEend

; --- D --- crsr left
LE4     cmp #$44    ; D ?
        bne LE5
	jsr clPending
        jsr GetNum  ; get argument
        lda EPar    ; EPar = 0...
        bne LE4c
        inc EPar    ; .. means 1        
LE4c    lda sCol    ; get crsr col        
        sec
        sbc EPar    ; col = col - left
        bpl LE4a    ; stop at left..
        lda #$00    ; ..margin
LE4a    tay         ; y is col
        ldx sRow    ; x is row
        jsr CPlot   ; set crsr
        jmp LEend

; --- m ---  font attributes
LE5     cmp #$6d    ; m ?
        beq LE5a
        jmp LE6	    ; too far to branch
LE5a    jsr GetNum
        pha         ; save nondigit char
	lda Font    ; font to A
        ldy EPar    ; parameter to Y
        ; -- 0 --
        bne LE5b    ; 0 ?
        tya         ; set font = vanilla
        jmp LE5nx
        ; -- 1 -- bold
LE5b    cpy #$01
        bne LE5c
        ora #$02    ; bit 1 = bold
        jmp LE5nx
        ; -- 4 -- underline
LE5c    cpy #$04
        bne LE5d
        ora #$04    ; bit 2 = underline
        jmp LE5nx
        ; -- 7 -- reverse
LE5d    cpy #$07
        bne LE5e
	ora #$01    ; bit 0 = reverse
        jmp LE5nx
LE5e    ; -- 30 - 37 --
        cpy #38     ; >= 38?
        bcs LE5g
        cpy #30     ; < 30?
        bcc LE5g
	tya
	sbc #30
	sta AFore
	lda Font
	ora #$80    ; bit 7 = fore
        jmp LE5nx
LE5g	; -- 40 - 47 --
        cpy #48     ; >= 48?
        bcs LE5nx
        cpy #40     ; < 40?
        bcc LE5nx
	tya
	sbc #40
	sta ABack
	lda Font
	ora #$40    ; bit 6 = back
LE5nx   ; -- next char --
	sta Font
	pla         ; get nondigit char
        cmp #$3b    ; is semicolon?
        beq LE5a    ; then next param
        ; -- set colour --
	lda Font
	; set foreground
	bpl LE5h    
	ldx AFore
	lda AColTab,x
	; avoid black or white foregr if
	; backgr is given and not black
	tax
	lsr	    ; 0(blk) or 1(wht)?
	bne LE5j    ; no -> keep fore
	bit Font    ; back? (bit 6 -> v)
	bvc LE5j    ; no -> keep fore
	lda ABack   ; back is black?
	bne LE5hh   ; no -> take back
LE5j	txa
	; ^^^ end of avoid ^^^
	ldx #$00    ; reverse off
        jmp LE5k 
LE5h	; set background
	bit Font    ; bit 6 -> v
	bvc LE5i
LE5hh	ldx ABack
	lda AColTab,x
	ldx #$01    ; reverse on
        jmp LE5k 
LE5i	; set ANSI font attributes
	ldx #$00    ; reverse off
	lsr	    ; reverse?
	bcc LE5l
	inx	    ; -> reverse on
LE5l	tay
        lda FontTable,y
LE5k    sta sColor  ; set colour
	stx Rvs	    ; set reverse mode
        jmp LEend

_cFontTable
FontTable    ; bits mean ul bo
.byt fVa, fBo         ; 00 01
.byt fUl, fUlBo       ; 10 11

AColTab
;ANSI 30  31  32  32  34  35 36  37 
;    blk red grn  ye blu mag cy  wh
.byt  0, 10,  5,  7, 14,  4,  3,  1

; --- K --- erase line
LE6     cmp #$4b      ; K ?
        bne LE7
        jsr GetNum    ; get parameter
        lda EPar      ; in A
        ; -- 0 -- crsr to end of line
        bne LE6b
        jsr ErEnLn    ; erase end line
        jmp LEend
        ; -- 1 -- begin to crsr
LE6b    cmp #$01
        bne LE6d
        jsr ErBeLn    ; erase beg line
        jmp LEend
        ; -- 2 -- whole line
LE6d    cmp #$02
        bne LE6e      ; par undefined 
        ldx sRow      ; line in X
        jsr ErLn      ; erase line
        sta sCrsrChar ; del char ..
                      ; ..under crsr
LE6e    jmp LEend        
        

; --- f --- same as H
LE7     cmp #$66
        bne LE8
        jmp LE7a      ; same as H

; --- H --- cursor position
LE8     cmp #$48
        bne LE9
LE7a	jsr clPending
        cpy #$01    ; no par means home
        bne LE8a
        ; -- home --
        ldx #$00
        ldy #$00
        jsr CPlot   ; set crsr
        jmp LEend
        ; -- row, col --
LE8a    jsr GetNum
        cmp #$3b    ; is ;?
        bne LE8d    ; no -> error
        ; -- prepare row --
        ldy EPar    ; get row
        bne LE8b    ; 0 means 1
        iny
LE8b    dey         ; line 1 -> line 0
        cpy #$19    ; >= 25?..
        bcs LE8d    ; ..error!
        sty xVector ; save row
        ; -- prepare col
        jsr GetNum
        ldy EPar    ; get col
        bne LE8c    ; 0 means 1
        iny
LE8c    dey         ; col 1 -> col 0        
        cpy #$28    ; >= 40?..
        bcs LE8d    ; ..error!        
        ldx xVector ; restore row to X
        jsr CPlot   ; set crsr
LE8d    jmp LEend

; --- J --- erase screen
LE9     cmp #$4a      ; J ?
        bne LE10
        jsr GetNum    ; get parameter
        lda EPar      ; in A
        ; -- 0 -- crsr to end
        bne LE9a
        jsr ErEnLn    ; del rest of line
        ldx sRow      ; get crsr line
LE9b    inx           ; next line
        cpx #$19      ; line 25?
        bcs LE9f      ; then end
        txa
        pha           ; save X
        jsr ErLn      ; erase line
        pla
        tax           ; restore X
        jmp LE9b      ; next line
        ; -- 1 -- beg of screen to crsr
LE9a    cmp #$01
        bne LE9e
        jsr ErBeLn    ; del start of ln
        ldx sRow      ; get crsr line
LE9c    dex           ; previous line
        bmi LE9f      ; neg line -> end
        txa
        pha           ; save X
        jsr ErLn      ; erase line
        pla
        tax           ; restore X
        jmp LE9c
        ; -- 2 -- del screen
LE9e    cmp #$02      ; unknown?
        bne LE9f      ; then ingnore
        ldx #$18      ; start at ln 24        
LE9d    txa
        pha           ; save X
        jsr ErLn      ; erase line
        pla
        tax           ; restore X
        dex           ; previous line
        bpl LE9d
LE9f    jmp LEend

; --- r ---  set scroll region                 
LE10    cmp #$72    ; r ?
        bne LE11
        ; -- prepare top --
        jsr GetNum
        cmp #$3b    ; is ;?
        bne LE10e   ; no -> error
        ldy EPar    ; get top
        dey         ; line 1 -> line 0
        cpy #$19    ; >=25?..
        bcs LE10e   ; ..error!
        sty xVector ; save top      
        ; -- prepare bottom --
        jsr GetNum
        ldy EPar    ; get bottom
        dey         ; line 1 -> line 0
        cpy #$19    ; >=25?..
        bcs LE10e   ; ..error! 
        sty yVector ; save bottom       
        ; -- validate lines --
        lda xVector ; restore top
        cmp yVector ; >= bottom?..
        bcs LE10e   ; ..error!
        sta SRS     ; top -> SRStart
        sty SRE     ; bottom -> SREnd
        ; -- home crsr
	jsr clPending
        ldx #$00
        ldy #$00
        jsr CPlot
LE10e   jmp LEend        

; --- l --- set crsr invisible
LE11	cmp #$6c    ; l ?
	bne LE12
	lda EBuf    ; first char ..
	cmp #$3f    ; .. is '?' ?
	bne LE11a
	inx	    ; at second char ..
	jsr GetNum
	lda EPar
	cmp #25	    ; .. 25 ?
	bne LE11a
	jsr COff    ; switch crsr off
	sta civis   ; mark invisible
LE11a	jmp LEend

; --- h --- set crsr visible
LE12	cmp #$68    ; h ?
	bne LE13
     	lda EBuf    ; first char ..
	cmp #$3f    ; ... is '?' ?
	bne LE12a
	inx	    ; at second char ..
	jsr GetNum  
	lda EPar
	cmp #25	    ; .. 25 ?
	bne LE12a
	lda #$00
	sta civis   ; mark visible
     	jsr COn     ; switch crsr off
LE12a	jmp LEend

LE13
; --- unknown esc seqence ---
LEend   lda #$00
        sta EBufL   ; reset esc buffer
        sta EMode   ; reset esc mode
        rts

; -------------------------------------
; GetNum - get decimal number from 
;          esc sequence
;
; params: esc sequence in EBuf
;         first index to process in X
; affects: A, X, Y
; return: number in EPar
;         first non digit char in  A
;	  next index to process in X
; -------------------------------------

GetNum  lda #$00    ; init value
        sta EPar
GN2     lda EBuf,x  ; get next char
        inx
        jsr TestD   ; digit?
        bcc GN1     ; no -> return
	tay	    ; digit to Y
        lda EPar
	jsr addDecDig
        sta EPar
        jmp GN2     ; next char
        
GN1     rts        

; *************************************
; *
; * outgoing data
; *
; *************************************

; -------------------------------------
; process outgoing key
;
; params: key in Y
; -------------------------------------
ProcOut 
        lda NumMode ; handle num mode
        bne ProcNum
        lda pta,y   ; PETSCII to ASCII
        beq POrts   ; ignore key
        cmp #$ff
        beq StrKey  ; send a string
        cmp #$fe
        beq CmdKey  ; command key
        jsr putRS
POrts   rts

; -------------------------------------
; handle num mode
;
; -------------------------------------
ProcNum
        tya         ; get char
        jsr TestD   ; digit?
        bcs PrN1
        jmp PNerr   ; no -> error

PrN1    and #$0f    ; digit to value
        jsr PutNum  ; put to display
        bcc PrNend  ;  no valid value
        jmp putRS   ;  send valid value

PrNend  rts

; -------------------------------------
; outgoing string 
;
; params: key in y
; -------------------------------------
ScrsrU .byt $1b, $5b, $41, $00 ; esc [ A
ScrsrD .byt $1b, $5b, $42, $00 ; esc [ B
ScrsrR .byt $1b, $5b, $43, $00 ; esc [ C
ScrsrL .byt $1b, $5b, $44, $00 ; esc [ D 

StrKey  tya         ; restore key

; --- crsr U ---
K1      cmp #$91    ; test crsr U
        bne K2
        ldx #<ScrsrU
        ldy #>ScrsrU
        jsr SendStr
        rts
; --- crsr L ---
K2      cmp #$9d    ; test crsr L
        bne K3
        ldx #<ScrsrL
        ldy #>ScrsrL
        jsr SendStr
K3      rts

; -------------------------------------
; outgoing command key
;
; -------------------------------------
CmdKey  tya         ; restore character

; --- crsr R ---
; ---   ^]   ---
; both events send $1d
        cmp #$1d
	bne C0
        lda #$04    ; test control Key
        bit ControlFlags
        beq crsrR   ; not pressed
        ; control ] is pressed
        tya         ; send ^]
        jsr putRS
	rts

	; crsr R 
crsrR   ldx #<ScrsrR
        ldy #>ScrsrR
        jsr SendStr
        rts

; --- crsr D ---
; ---   ^Q   ---
; both events send char $11
C0      cmp #$11    ;^Q / crsr down
        bne C1
        lda #$04    ; test control Key
        bit ControlFlags
        beq crsrD   ; not pressed
        ; control Q is pressed
        tya         ; send ^Q
        jsr putRS
        rts
        
        ; crsr down is pressed        
crsrD   ldx #<ScrsrD
        ldy #>ScrsrD
        jsr SendStr
        rts

; --- C=D ---
; dump mode
C1      cmp #$ac    ; C=D
        bne C2
        lda #$00    ; clear esc mode
        sta EMode
        lda DMode   ; toggle dump mode
        eor #$01
        sta DMode
	beq C1a     ; dump switched off?
	jsr XCon    ; -> skip 
C1a     ldx #<SDump ; print user info
        ldy #>SDump
        jsr CPrnStr
	lda DMode   ; dump on?
	bne C1b	    ; -> skip
	jsr XCoff
C1b     rts
SDump   .asc "-dUMP MODE-"
        .byt $00

; --- C=H ---
; print help
C2      cmp #$b4    ; C=H
	bne C3
	jmp Help

; --- C=A ---
; get asci char
C3      cmp #$b0    ; C=A
        bne C4
        jmp OpenNum

; --- C=R ---
; receive xmodem
C4      cmp #$b2    ; C=R
        bne C5
        jmp Xrec

; --- C=L ---
; list directory
C5      cmp #$b6    ; C=L
        bne C6
        jmp XDir

; --- C=S ---
; send xmodem
C6      cmp #$ae    ; C=S
        bne C7
        jmp Xsend

; --- C=F ---
; send floppy command
C7      cmp #$bb    ; C=F
        bne C8
        jmp Xcommand

; --- HOME key ---
; ---    ^S    ---
; both events send char $13
C8      cmp #$13    ;^S / HOME
        bne C9
        lda #$04    ; test control Key
        bit ControlFlags
        beq C8Home  ; not pressed
        ; control S is pressed
        tya         ; send ^S
        jsr putRS
        rts
        
        ; send TAB
C8Home  lda #$09
        jsr putRS
        rts
 
; --- HOME key ---
; ---    ^T    ---
; both events send char $14
C9      cmp #$14    ;^S / DEL
        bne C10 
        lda #$04    ; test control Key
        bit ControlFlags
        beq C9Del   ; not pressed
        ; control T is pressed
        tya         ; send ^T
        jsr putRS
        rts
        
        ; send TAB
C9Del   lda #$08
        jsr putRS
        rts

; --- C=Q ---
; quit CaTer
C10     cmp #$ab    ; C=Q
	bne C11
	; del last char of 'CBM80'
	lda #$00
	sta $8008
	; cold start
	jmp ($fffc)

; --- C=C ---
; Config menu
C11     cmp #$bc    ; C=C
        bne C12
	jmp ConfigMenu

; --- unknown C=-Key ---
C12     rts

; -------------------------------------
; ConfigMenu 
;
; called from outgoing data loop
; returns with rts
; -------------------------------------
ConfigMenu
	jsr XCon
CmStart	; --- print menu ---
	; printf floppy
	ldx #<CfgMen2
	ldy #>CfgMen2
	jsr CPrnStr
	lda Floppy
	sta xVector
	lda #$00
	sta xVector+1
	jsr IntStr
	ldx #<DecNum
	ldy #>DecNum
	jsr CPrnStrNL
	; print baud rate
	ldx #<CfgMen1
	ldy #>CfgMen1
	jsr CPrnStr
	ldx Baud
	cpx #$0e    ; IEC
	beq Cm1
	lda CfgBaudList,x
	sta xVector
	lda CfgBaudList+1,x
	sta xVector+1
	jsr IntStr
	ldx #<DecNum
	ldy #>DecNum
	jmp Cm2
Cm1 	ldx #<CfgBaudIec
    	ldy #>CfgBaudIec
Cm2     jsr CPrnStrNL
	; printf iec device
	ldx #<CfgMen3
	ldy #>CfgMen3
	jsr CPrnStr
	lda IECdevice
	sta xVector
	lda #$00
	sta xVector+1
	jsr IntStr
	ldx #<DecNum
	ldy #>DecNum
	jsr CPrnStrNL
	; printf iec string
	ldx #<CfgMen4
	ldy #>CfgMen4
	jsr CPrnStr
	ldx #<IECstring
	ldy #>IECstring
	jsr CPrnStrNL
	; print exit
	ldx #<CfgMenX
	ldy #>CfgMenX
	jsr CPrnStrNL
	; --- read char from kbd ---
CmKbd   jsr WaitChr
	; (X) exit
	cmp #$58      ; X
	bne Cm_0
	jmp Cmend
Cm_0	; (F) floppy
	cmp #$46      ; F
	bne Cm_1
	ldx #<CfgMenEn
	ldy #>CfgMenEn
	jsr CPrnStr  
	jsr XkbdNumNL
	bcs Cm3       ; num invalid?
	jmp CmReError ; yes -> error
Cm3	ldx #4
	ldy #30
	jsr NumRange
	bcs Cm4       ; out of range?
	jmp CmReError ; yes -> error
Cm4	sta Floppy
	jmp CmReStart
Cm_1	; (B) baud rate
	cmp #$42      ; B
	bne Cm_2
	ldx #<CfgMenBd
	ldy #>CfgMenBd
	jsr CPrnStrNL
	jsr WaitChr ; get baud from user
	ldx #$31    ; "1"
	ldy #$35    ; "5"
	jsr NumRange
	bcc CmReError
	pha
	jsr CloseCh ; close rs232
	pla
	sec	    ; digit to baud:
	sbc #$31    ; "1".."5" -> 0..4
	asl	    ; -> 0,2,4,6,8
	adc #$06    ; -> 6,8,a,c,e
	sta Baud
	jsr InitCh  ; open rs232
	jmp CmReStart
Cm_2	; (I) iec device
	cmp #$49    ; I
	bne Cm_3
	ldx #<CfgMenEn
	ldy #>CfgMenEn
	jsr CPrnStr  
	jsr XkbdNumNL
	bcc CmReError ; invalid -> error
   	ldx #4
	ldy #30
	jsr NumRange
	bcc CmReError ; invalid -> error
        sta IECdevice
Cm6	lda Baud      ; IEC active?
	cmp #$0e
	bne Cm5	      ; then:
	jsr CloseCh   ; close rs232
	jsr InitCh    ; open rs232
Cm5	jmp CmReStart
Cm_3    ; (S) iec string
	cmp #$53      ; S
	bne Cm_4
	ldx #<CfgMenEs
	ldy #>CfgMenEs
	jsr CPrnStr  
	jsr XkbdR
	stx IECstrLen ; store length
	lda #<XkbdBuf 
	sta xVector
	lda #>XkbdBuf
	sta xVector+1 ; copy input ..
	lda #<IECstring
	sta yVector
	lda #>IECstring
	sta yVector+1 ; .. to IECstring
	jsr strcpy
	jmp Cm6	  ; reopen iec like (I)
Cm_4    ; (?) unknown
    	jmp CmKbd     ; unknown

Cmend 	jsr XCoff
	rts

	; --- restart on error ---
CmReError
	lda #RBuzz    ; buzzer request
	sta BellR
	; --- restart on success ---
CmReStart
	jsr CR
	jsr LF
	jmp CmStart

CfgMen1 .asc "b bAUD RATE  : ", $00
CfgMen2 .asc "f dISK DEVICE: ", $00
CfgMen3 .asc "i iec  DEVICE: ", $00
CfgMen4 .asc "s iec  STRING: ", $00
CfgMenX .asc "x eXIT MENU", $00
CfgMenEn .asc "eNTER NUMBER: ", $00
CfgMenEs .asc "eNTER sTING: ", $00
             ;".........1.........2.........3.........4"
CfgMenBd .asc "1:300  2:1200  3:2400  4:9600  5:iec", $00

CfgBaudList_ .word 300, 1200, 2400, 9600
CfgBaudList = CfgBaudList_ - 6
CfgBaudIec   .asc "iec ", $00

; -------------------------------------
; Help - print help screen
;
; calledom outgoing data loop
; returns with rts
; -------------------------------------
Help	jsr XCon
	ldx #<HelpStr
	ldy #>HelpStr
	jsr CPrnStrNL
	jsr XCoff
	rts

HelpStr ;".........1.........2.........3.........4"
.asc     "c=l fLOPPY LIST    c=r xmodem RECEIVE   "
.asc	 "c=f fLOPPY COMMAND c=s xmodem SEND      "
.asc	 "c=a sEND ascii     c=d dUMP MODE        "
.asc     "c=q qUIT cAtER     c=c cONFIG MENU"
.byt $00

; *************************************
; *
; * screen handling
; *
; *************************************

; Variables controlling the cursor
;
;                             COff
;                     Kernal  COon Plot
; sCrsrOn	$cc	r      x
; sCrsrCnt	$cd	r/w    x
; sCrsrPhase	$cf	r/w    x
; sCol		$d3	r           x
; sRow		$d6                 x
; sLinePtr	$d1	r           x
; sLineColPtr	$f3	r/w         x
; sCrsrChar	$ce	r/w    x
; sCrsrCol	$0287	r/w    x
; sColor	$0286	r

; Cursor blink phases:
; on (sCrsrPhase = $01)
;   char on screen = revers of sCrsrChar
;   color on screen = sColor
; off (sCrsrPhase = $00)
;   char on screen = sCrsrChar
;   color on screen = sCrsrCol

; -------------------------------------
; COff - switch cursor off
;
; affects: none
;
; Switch cursor off and restore char.
; This has to be done before every crsr
; movement.
; After movement COn has to be called.
; -------------------------------------
COff    pha            ; save registers
        tya
        pha             

	lda civis      ; invisible?
	bne CO2	       ; -> do nothing
	ldy #$02       ; prevent soon..
	sty sCrsrCnt   ; ..blink
        sty sCrsrOn    ; crsr off    
        lda sCrsrPhase ; crsr revers?
        beq CO2        ; no -> return
        dey            ; normal phase
        sty sCrsrPhase        
        ldy sCol       ; get column
        lda sCrsrChar  ; restore char
        sta (sLinePtr),y
        lda sCrsrCol   ; restore colour
        sta (sLineColPtr),y

CO2     pla         ; restore registers
        tay
        pla
        rts

; -------------------------------------
; COn - switch crsr on
;
; affects: none
;
; opposite of COff
; -------------------------------------
COn     pha
        tya
        pha

	lda civis	 ; invisible?
	bne COn1	 ; -> do nothing
        ldy sCol	 ; get column
        lda (sLinePtr),y ; save chr
        sta sCrsrChar 
        eor #$80         ; reverse char
        sta (sLinePtr),y        
        lda (sLineColPtr),y; save colour
        sta sCrsrCol
        lda sColor     ; set crsr colour
        sta (sLineColPtr),y
        lda #$14       ; set counter...
        sta sCrsrCnt   ; ... to max
	ldy #$01       ; set rvs phase
        sty sCrsrPhase
        dey            ; cursor on
        sty sCrsrOn 
        
COn1    pla
        tay
        pla
        rts

; -------------------------------------
; CPlot - move cursor
;
; params: coumn in Y
;         line in X
; affects: A, X, Y
;
; The crsr ist turned off during 
; operation (COff - COn)
; -------------------------------------
CPlot   jsr COff
	jsr Plot
        jsr COn        
        rts

; -------------------------------------
; Plot - move cursor
;
; params: coumn in Y
;         line in X
; affects: A, X, Y
; -------------------------------------
Plot    stx sRow     ; set row
        sty sCol     ; set col
        jsr SLV
        ldx xVector ; set screen line
        ldy xVector+1
        stx sLinePtr
        sty sLinePtr+1
        ldx yVector ; set color line
        ldy yVector+1
        stx sLineColPtr
        sty sLineColPtr+1
	rts

; -------------------------------------
; CPrnChr - print char to screen
;
; params: chr in A, $ff means no output
; affects: A
; uses: xVector
;
; The crsr ist turned off during 
; operation (COff - COn)
; -------------------------------------
CPrnChr jsr COff
        jsr PrnChr
        jsr COn
        rts

; -------------------------------------
; PrnChr - print char (petscii) to screen
;
; params: chr in A, $80 means no output
; affects: A
; uses: xVector
;
; Rem: The char π/~ is read from 
; keyboard as $de but from screen
; (basic code, via input) as $ff.
; -------------------------------------
PrnChr  sta xVector ; save char
        txa         ; save registers
        pha
        tya
        pha
        lda xVector ; restore char

	; PETSCII to ScreenCode (SC)
	; -- $ff -- π/~ from keybd
	cmp #$ff      ; warn: reverse
	bne PC_1
	lda #$de
	jmp PCrvs
	; -- $e0-$fe -- upper graphics
PC_1    cmp #$e0      ; no output
        bcc PC0
        jmp PCend
        ; -- $c0-$df -- capital letters
PC0	cmp #$c0
	bcc PC1
	and #$7f
	jmp PCrvs
	; -- $a0-$bf -- graphics/latin-9
PC1	cmp #$a0
	bcc PC2
	and #$3f
	ora #$40
	jmp PCrvs
	; -- $80-$9f -- upper control
PC2	cmp #$80      ; no output
	bcc PC3
	jmp PCend
	; -- $60-$7f -- lower capital
PC3	cmp #$60 ; for string constants
	bcc PC4
	and #$5f
	jmp PCrvs
	; -- $40-$5f -- small letters
PC4	cmp #$40
	bcc PC5
	and #$1f
	jmp PCrvs
	; -- $20-$3f -- numbers
PC5	cmp #$20
	bcc PC6
	jmp PCrvs
	; -- $00-$1f -- lower control
PC6	jmp PCend     ; no output

	; -- handle reverse mode --
PCrvs   ldx Rvs     ; reverse mode?
        beq PCput
        eor #$80    ; reverse char

PCput   ldx lbPending	; need new line?
	beq PC7		; no -> skip
	ldx #$00	; clear pending
	stx lbPending
        jsr NewLn
PC7	ldy sCol        ; get crsr col
        sta (sLinePtr),y; char to screen
        lda sColor      ; get colour
        sta (sLineColPtr),y ; set colour
        
	; -- move on crsr --
        cpy #$27        ; col = 39?
	bne PC8	        ; no -> skip
	lda #$ff        ; yes -> set..
	sta lbPending   ; ..pending
	jmp PCend
PC8     iny             ; move on crsr
        sty sCol
        
PCend   pla         ; restore registers
        tay
        pla
        tax
        rts

; -------------------------------------
; NewLn - move crsr to next line
;
; affects: X, Y
;
; --- INTERNAL --- 
; subtask of PrnChr
; -------------------------------------
NewLn   pha	    ; save char
	ldx sRow    ; get crsr row
        cpx SRE     ; end of scroll reg?
        beq NL1     ; yes -> branche
        cpx #$18    ; line 24?
        beq NLend   ; yes -> crsr stays
	; --- normal wrap ---
        inx         ; increase line
        ldy #$00    ; begin of line
        jsr Plot
        jmp NLend
	; --- scroll up ---        
NL1     jsr UScrl
        ldy #$00    ; begin of line
        sty sCol
NLend   pla	    ; restore char
	rts

; -------------------------------------
; DEL - move crsr to the left and 
;       delete char
;
; Can move through left margin.
;
; affects: A, X, Y
; -------------------------------------
DEL	jsr COff
	ldy sCol
	lda lbPending
	beq DEL1
	; pending
	jsr clPending
	jmp DELe

DEL1	dey
	bmi DEL2
	; middle of line
        sty sCol
	jmp DELe

DEL2	; first col
	ldx sRow
	beq DELee ; odd: top left corner
	dex
	ldy #39
	jsr Plot
	ldy sCol

DELe	lda #" "  ; clear char
	sta (sLinePtr),y
DELee	jsr COn
	rts

; -------------------------------------
; clPending - clear pending flag
; 
; affects: none
; 
; Must be called in all crsr movement
; commands.
; -------------------------------------
clPending
	pha
	lda #$00
	sta lbPending
	pla
	rts

; -------------------------------------
; CUScrl - scroll up scrollregion
;
; affects: A, X, Y

; The crsr ist turned off during 
; operation (COff - COn)
; -------------------------------------
CUScrl	jsr COff
	jsr UScrl
	jsr COn
	rts

; -------------------------------------
; UScrl - scroll up scrollregion
;
; affects: A, X, Y
; uses: xVector, yVector, 
;       zVector, wVector
; -------------------------------------
UScrl   ldx SRS     ; get first line
	; --- scroll one line ---
US1     ; -- new line: --
        ; -- zVector and wVector --
        jsr SLV
        lda xVector ; screen line
        ldy xVector+1
        sta zVector
        sty zVector+1
        lda yVector ; colour line
        ldy yVector+1
        sta wVector
        sty wVector+1
        ; -- old line: --
        ; -- xVector and yVector
        inx             ; old line
        jsr SLV
        ; -- copy chars and colours --
        ldy #$27        ; col 39
US2     lda (xVector),y ; copy char
        sta (zVector),y
        lda (yVector),y ; copy colour
        sta (wVector),y
        dey
        bpl US2
        cpx SRE         ; last line?
        bne US1         ; no -> go on
        jsr ErLn_       ; del last line
        
        rts

; -------------------------------------
; CDScrl - scroll down scrollregion
;
; affects: A, X, Y
;
; The crsr ist turned off during 
; operation (COff - COn)
; -------------------------------------
CDScrl	jsr COff
	jsr DScrl
	jsr COn
	rts

; -------------------------------------
; DScrl - scroll down scrollregion
;
; affects: A, X, Y
; uses: xVector, yVector, 
;       zVector, wVector
; -------------------------------------
DScrl   ldx SRE     ; get last line
	; --- scroll one line ---
DS1     ; -- new line: --
        ; -- zVector and wVector --
        jsr SLV
        lda xVector ; screen line
        ldy xVector+1
        sta zVector
        sty zVector+1
        lda yVector ; colour line
        ldy yVector+1
        sta wVector
        sty wVector+1
        ; -- old line: --
        ; -- xVector and yVector
        dex             ; old line
        jsr SLV
        ; -- copy chars ond colours --
        ldy #$27        ; col 39
DS2     lda (xVector),y ; copy char
        sta (zVector),y
        lda (yVector),y ; copy colour
        sta (wVector),y
        dey
        bpl DS2
        cpx SRS         ; first line?
        bne DS1         ; no -> go on
        jsr ErLn_       ; del first line
        
        rts

; -------------------------------------
; CPrnStrNL - print string to sceen,
;             followed by CR NL
;
; string: chars, terminated by $00
; params: string ptr lo in X
;         string ptr hi in y
; affects: A, X, Y
;
; The string must be smaller than 
; 255 chrs.
; The crsr ist turned off during 
; operation (COff - COn)
; -------------------------------------
CPrnStrNL
	jsr CPrnStr
	jsr CR
	jsr LF
	rts

; -------------------------------------
; CPrnStr - print string to screen
;
; string: chars, terminated by $00
; params: string ptr lo in X
;         string ptr hi in y
; affects: A
;
; The string must be smaller than 
; 255 chrs.
; The crsr ist turned off during 
; operation (COff - COn)
; -------------------------------------
CPrnStr stx vVector   ; store string ptr
        sty vVector+1
        jsr COff
        
        ldy #$00
L1      lda (vVector),y
        beq L2      ; string ends at $00
        jsr PrnChr
	; -- put char to screen --
        iny
        jmp L1

L2      jsr COn
        rts

; -------------------------------------
; ErLn - erase screen line
;
; params: line number in X
; affects: A, X, Y
; return: $20 (space) in A
;
; For internal use:
; ErLn_ needs line ptr in xVector 
; and line colot ptr in yVector
; -------------------------------------
ErLn    jsr SLV ; line start in xVector
                ; col  start in yVector

	; -- erase chars --
ErLn_   ldy #$27      ; col 39
        lda #$20      ; load space
EL1     sta (xVector),y ; clear char
        dey 
        bpl EL1

	; -- set colour --
        ldy #$27      ; col 39
        lda #fVa      ; load vanilla
EL2     sta (yVector),y ; clear char
        dey 
        bpl EL2
        
        rts

; -------------------------------------
; ErEnLn - erase to end of line
;
; affects: A, X, Y
;
; erase screen line from crsr to end of line
; -------------------------------------
ErEnLn
	; -- erase chars --
        ldy sCol      ; get crsr col
        lda #$20      ; load space
EEL1    sta (sLinePtr),y ; clear char
        iny
        cpy #$28      ; pos 40?
        bne EEL1      ; next char
        sta sCrsrChar ; del char ..
                      ; ..under crsr
	; -- set colour --
        ldy sCol      ; get crsr col
        lda #fVa      ; load vanilla
EEL2    sta (sLineColPtr),y ; set colour
        iny
        cpy #$28      ; pos 40?
        bne EEL2      ; next char
        
        rts

; -------------------------------------
; ErBeLn - erase from begin of line
;
; affects: A, X, Y
;
; erase screen line up to crsr
; -------------------------------------
ErBeLn
	; -- erase chars --
        ldy sCol      ; get crsr col
        lda #$20      ; load space
EBL1    sta (sLinePtr),y ; clear char
        dey
        bpl EBL1      ; pos>=0 -> next
        sta sCrsrChar ; del char ..
                      ; ..under crsr
	; -- set colour --
        ldy sCol      ; get crsr col
        lda #fVa      ; load vanilla
EBL2    sta (sLineColPtr),y ; clear char
        dey
        bpl EBL2      ; pos>=0 -> next
        
        rts

; -------------------------------------
; SLV - set line vectors
; --- INTERNAL ---
;
; params: line no in X
; affects: A, Y
; return: screen line ptr in xVector
;         screen color ptr in yVector
; -------------------------------------
SLV     lda $ecf0,x   ; get lo byte
        sta xVector
        sta yVector
        ; determin hi byte
        ldy #$04      ; hi byte
        cpx #$07      ; line < 7?
        bcc SLV1
        iny           
        cpx #$0d      ; line < 13?
        bcc SLV1
        iny
        cpx #$14      ; line < 20?
        bcc SLV1
        iny           ; line 20-24
SLV1    sty xVector+1
        tya
        clc           ; colour RAM =
        adc #$d4      ; video RAM + d4
        sta yVector+1
        
        rts

; -------------------------------------
; XCon  - set yellow font, next line
; XCoff - restore font, not next line
;
; affects: A, X, Y
;
; Switch to another font indicating 
; cater is speaking.
; -------------------------------------
XCon    lda sColor  ; save present..
        sta XCrsr   ; ..colour
_cColEx lda #colExtra
        sta sColor
        jsr CR      ; next screen line
        jsr LF
	rts
        
XCoff   lda XCrsr   ; get colour
        sta sColor
        rts

; *************************************
; *
; * Sprite Number panel
; *
; *************************************

; -------------------------------------
; OpenNum
;
; affects: a
; -------------------------------------
OpenNum
        lda #$01
        sta NumMode  ; Num mode on
        ; digits to dot
        lda #SpriteBaseVIC+11
        sta SpritePtr0
        sta SpritePtr1
        sta SpritePtr2        
        lda #$00
        sta NumClose ; request off
        sta NumVal   ; Value to 0
	lda #$03     ; expect 3 digits
        sta NumSize
        ; sprites on
        lda #$0f
        sta $d015
        
        rts

; -------------------------------------
; PutNum
;
; put number in A to buffer
; affects: A, X, Y
; return: a=value, c=valid
; -------------------------------------
PutNum
	; --- rotate old digits ---
        ldy SpritePtr1
        sty SpritePtr0
        ldy SpritePtr2
        sty SpritePtr1     
        ; --- print new digit ---
        tay           ; new digit in Y
        clc         
        adc #SpriteBaseVIC
        sta SpritePtr2             
	; --- update value ---
        lda NumVal    ; old val in A
	jsr addDecDig
	bcs PNerr
        sta NumVal    
        ; --- three digits? ---
	sec	      ; assume finished
        dec NumSize   ; really finished?
        beq PNend     ; yes -> branch
	clc	      ; no
        rts        

; -------------------------------------
; exit num mode with error
PNerr   
        lda #RBuzz    ; buzzer request
        sta BellR
        clc           ; no valid output

; -------------------------------------
; exit num mode successfully
PNend   ldx #LNum     ; Number off
        stx NumClose
        ldx #$00      ; num mode off
        stx NumMode
        rts

; -------------------------------------
; Init Sprites
;
; -------------------------------------
InitSprites

Digit0 = FontROM + $180
Sprite0 = SpriteBase
        ldx #<Digit0
        ldy #>Digit0
        stx xVector
        sty xVector+1
        ldx #<Sprite0
        ldy #>Sprite0
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit1 =  Digit0 + 8
Sprite1 = Sprite0 + $40
        ldx #< Digit1
        ldy #> Digit1
        stx xVector
        sty xVector+1
        ldx #<Sprite1
        ldy #>Sprite1
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit2 =  Digit1 + 8
Sprite2 = Sprite1 + $40
        ldx #< Digit2
        ldy #> Digit2
        stx xVector
        sty xVector+1
        ldx #<Sprite2
        ldy #>Sprite2
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit3 =  Digit2 + 8
Sprite3 = Sprite2 + $40
        ldx #< Digit3
        ldy #> Digit3
        stx xVector
        sty xVector+1
        ldx #<Sprite3
        ldy #>Sprite3
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit4 =  Digit3 + 8
Sprite4 = Sprite3 + $40
        ldx #< Digit4
        ldy #> Digit4
        stx xVector
        sty xVector+1
        ldx #<Sprite4
        ldy #>Sprite4
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit5 =  Digit4 + 8
Sprite5 = Sprite4 + $40
        ldx #< Digit5
        ldy #> Digit5
        stx xVector
        sty xVector+1
        ldx #<Sprite5
        ldy #>Sprite5
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit6 =  Digit5 + 8
Sprite6 = Sprite5 + $40
        ldx #< Digit6
        ldy #> Digit6
        stx xVector
        sty xVector+1
        ldx #<Sprite6
        ldy #>Sprite6
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit7 =  Digit6 + 8
Sprite7 = Sprite6 + $40
        ldx #< Digit7
        ldy #> Digit7
        stx xVector
        sty xVector+1
        ldx #<Sprite7
        ldy #>Sprite7
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit8 =  Digit7 + 8
Sprite8 = Sprite7 + $40
        ldx #< Digit8
        ldy #> Digit8
        stx xVector
        sty xVector+1
        ldx #<Sprite8
        ldy #>Sprite8
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
Digit9 =  Digit8 + 8
Sprite9 = Sprite8 + $40
        ldx #< Digit9
        ldy #> Digit9
        stx xVector
        sty xVector+1
        ldx #<Sprite9
        ldy #>Sprite9
        stx yVector
        sty yVector+1
        jsr ISCopyNumber
; background sprite
Sprite10 = Sprite9 + $40
        ldx #<Sprite10
        ldy #>Sprite10
        stx yVector
        sty yVector+1
        ; --- 27 filled bytes ---
        lda #$ff
        ldy #$00
ISbs    sta (yVector),y
        iny
        cpy #27
        bne ISbs
        ; --- 39 empty bytes ---
        lda #$00
ISbs1   sta (yVector),y
        iny
        cpy #$40  ; 64 bytes over all
        bne ISbs1
; dot sprite
Sprite11 = Sprite10 + $40
        ldx #<Sprite11
        ldy #>Sprite11
        stx yVector
        sty yVector+1
        ; --- clear sprite ---
        lda #$00
        ldy #$40
ISds1   sta (yVector),y
        dey
        bpl ISds1
        ; --- the dot ---
        lda #$18
        ldy #9
        sta (yVector),y
        iny
        iny
        iny
        sta (yVector),y
; VIC Registers
        ; --- y coordinate ---
        ldx #55
        stx $d001
        stx $d003
        stx $d005
        dex
        stx $d007
        ; --- x coordinate ---
        lda #60
        clc
        sta $d006
        sta $d000
        adc #$08
        sta $d002
        adc #$08
        sta $d004
        ; --- x 9th bit ---
        lda #$0f
        sta $d010
        ; --- colours ---
_cColNum ldx #colNumber
        stx $d027
        stx $d028 
        stx $d029
_cBgNum
        ldx #bgNumber
        stx $d02a 
        ; --- background pointer ---
        ldx #SpriteBaseVIC+10
        stx SpritePtr3
        
        rts        

ISCopyNumber
; copies number from char rom 
; to sprite area
; xVector = start of char
; yVector = start of sprite
        ldx #$00    ; lda () not valid
        ldy #$00
ISCN1   lda (xVector),y ; get bit line
        sta (yVector,x) ; store bit line
        txa             ; 2 empty bytes
        inc yVector
        sta (yVector,x)
        inc yVector
        sta (yVector,x)
        inc yVector     ; next sprite ln
        iny             ; next char line
        cpy #$08        ; 8 lines / char
        bne ISCN1
        ; 13 empty lines = 39 bytes
        ldy #39
ISCN2   sta (yVector),y
        dey
        bpl ISCN2
        
        rts

; *************************************
; *
; * Init routines
; *
; *************************************

; -------------------------------------
; init memory
;
; -------------------------------------
InitVar lda #$00
        sta DMode
        sta EMode
        sta EBufL
        sta SRS
        sta Font
        sta SaveF
        sta SaveR
        sta SaveRow
        sta SaveCol
        sta Xmodem
	sta lbPending
	sta civis
        
        lda #$18    ; last line
        sta SRE     ; = 24
_cFln	lda #Fln
	sta Floppy
_cBaud  lda #InitBaud
        sta Baud
	; IEC connection
     	lda #InitIECdev
	sta IECdevice
	lda #24
	sta IECstrLen
	lda #<InitIECstr
	sta xVector
	lda #>InitIECstr
	sta xVector+1
	lda #<IECstring
	sta yVector
	lda #>IECstring
	sta yVector+1
	jsr strcpy
        
        rts

InitIECstr ; 24 chars
	.asc "TCP:LOCALHOST:TELNET,X,N", $00

; -------------------------------------
; Init char
;
; at the end the font is apended.
; it is copied to $2000 and enabled

; uses: xVector, yVector, zVector
; -------------------------------------

; begin of reverse chars
Reverse = Char + $0400

InitChar  
        sei
        ldx #<FontROM ; Font in zVector
        ldy #>FontROM
        stx zVector
        sty zVector+1
        ldx #<Char    ; Char in yVector
        ldy #>Char
        stx yVector
        sty yVector+1
        ldx #<Reverse ; Revrs in xVector
        ldy #>Reverse
        stx xVector
        sty xVector+1

; copy font        
        ldx #$04      ; copy 4 pages        
        ldy #$00      
IC1     lda (zVector),y
        sta (yVector),y
        eor #$ff      ; reverse char
        sta (xVector),y
        iny
        bne IC1
        ; switch to next page
        inc zVector+1
        inc yVector+1
        inc xVector+1
        dex
        bne IC1         

; enable font
        lda $d018
        and #$f1
        ora #$09
        sta $d018
        cli
        rts        

; -------------------------------------
; Init Bell
;
; -------------------------------------

Sid=$d400

InitBell         
        ; --- voice 1 ---
        ; 500 Hz
        ; ADSR = $00 $f9
        ldx #$c4      ; set freq
        ldy #$20
        stx Sid+0
        sty Sid+1
        ldx #$00      ; set ADSR
        ldy #$f9
        stx Sid+5
        sty Sid+6
	; waveform is set in IRQ, sid+4
	Bell_c1 = $14 ; triang, ringmod

        ; --- voice 2 ---
        ; 200 Hz
        ; ADSR = $00 $20
        ldx #$0d      ; set freq
        ldy #$1b
        stx Sid+7
        sty Sid+8
        ldx #$00      ; set duty cycl
        ldy #$08      ; = 50 %
        stx Sid+9
        sty Sid+10
        ldx #$00      ; set ADSR
        ldy #$20
        stx Sid+12
        sty Sid+13
	; waveform is set in IRQ, sid+4
	Bell_c2 = $40 ; rect

        ; --- voice 3 ---
        ; 1400 Hz
        ; ringmodulation for voice 1
        ldx #$5b    ; set freq
        ldy #$c0
        stx Sid+14
        sty Sid+15

        ; --- filter ---
        ; 4000 Hz
        ; resonance 0, voices 1 and 2
        ; low pass, volume max
        ldx #$00      ; set freq
        ldy #$55
        stx Sid+21
        sty Sid+22
        ldx #$03      ; set res & voice
        stx Sid+23
	; volume and type in IRQ, sid+24
	Bell_vol = $1f
        
IBrts   rts

; -------------------------------------
; Init Screen
;
; -------------------------------------

; max 6 lines"                   \/                   "
istr    .asc "    ---+++#### c A t E R ####+++---     "
        .asc "                 v 1.1        18.03.2013"
        .asc "   (c) 2001 - 2013 lARS sTOLLENWERK     "
        .asc "    dISTRIBUTED UNDER THE TERMS OF      "
        .asc " gnu gENERAL pUBLIC lICENSE 2 OR LATER  "
        .asc "             hELP WITH c=h              "
        .byt $00

InitScr
	; --- set background ---
_cBorderCol
        lda #bgocolour
        sta $d020
_cBgCol lda #bgcolour
        sta $d021
	; --- disable Shift C= ---
        lda #$80
        sta ShiftCFlag
	; --- erase screen ---
        ldx #$18      ; start at ln 24        
IS1     txa
        pha           ; save X
        jsr ErLn      ; erase line
        pla
        tax           ; restore X
        dex           ; previous line
        bpl IS1
	; --- print initstring ---
        lda #fBo      ; load bold
        sta sColor
        ldx #<istr
        ldy #>istr
        jsr CPrnStrNL
        lda #fVa     ; load vanilla
        sta sColor
	; --- crsr on ---
        jsr COff
        jsr COn
	; --- put crsr ---
        jsr LF

        rts

; *************************************
; *                                   
; * IRQ routine             
; *                                   
; * handles Bell and keyboard
; *                                   
; *************************************

; Length of kept NumPanel in 1/50s
LNum = $50

IRQ
	; --- NumPanel ---
        ldx NumClose
        beq IRQbb     ; $00 -> no req.
        dex           ; count down
        stx NumClose
        bne IRQbb     ; $00?
        stx $d015     ; -> sprites off
IRQbb  	; --- Bell & Buzzer
        ldx BellR     ; handle bell
        beq IRQe      ; $00 -> no req.
        bmi IRQbell   ; neg -> new req.
        ; --- hold tone ---        
	lda #Bell_c1  ; voice 1 off
	sta Sid+4
        dex           ; count down
        stx BellR
	bne IRQe      ; time over?
	stx Sid+24    ; -> vol off
	lda #Bell_c2  ; -> voice 2 off
	sta Sid+11
        jmp IRQe
IRQbell ; --- start tone ---
        lda #Bell_vol
        sta Sid+24
	txa
	lsr           ; $81?
        bcs IRQ1      ; -> bell  
        ; start buzzer
        lda #Bell_c2+1
        sta Sid+11    ; voice 2
	lda #12	      ; timer 200 ms
	jmp IRQ2
IRQ1    ; start bell
        lda #Bell_c1+1
        sta Sid+4     ; voice 1
	lda #45	      ; timer 750 ms
IRQ2	sta BellR
IRQe    ; --- to kernal IRQ ---
        jmp $ea31

; Task of the original Kernal IRQ 
; at $ea31
;
; $ffea: count time in $a0, $a1, $a2
;        ClockH, ClockM, ClockL
; $f6bc: read RUN/STOP key, store in $91
; $ea34: blink cursor
; $ea61: motor datasette
; $ea87: read keyboard

; -------------------------------------
; init IRQ
;
; -------------------------------------
InitIRQ sei
        lda #<IRQ      ; set IRQ to own
        sta IRQ_Vector ; routine
        lda #>IRQ
        sta IRQ_Vector+1
        cli
        rts

; vim: set syntax=asm6502 :
